<?php
// vim: set noexpandtab tabstop=2 softtabstop=2 shiftwidth=2:

/**
 * TMUDataFetcher - Fetch TMU ladder/zones stats for a login
 * Created by Xymph <tmn@gamers.org>
 * Inspired by TMNDataFetcher
 *
 * v1.2: Bundle basic info & ladder stats requests into one XML block;
 *       add delimiter to number of zone's players requests
 * v1.1: Add $zoneInfo option and return $zonestats
 * v1.0: Initial release
 */
class TMUDataFetcher {

	var $version, $zones,
		$login, $nickname, $location, $worldrank, $totalplayers,
		$points, $lastscore, $wins, $losses, $draws, $envstats,
		$zonestats;

	/**
	 * Fetches a hell lot of data about a TMU login
	 *
	 * @param String $login
	 *        The TMU login to search for
	 * @param Boolean $zoneInfo
	 *        If true, the script also searches the login's zone ladders
	 * @return TMUDataFetcher
	 *        $envstats is an array with stats for each of the 7 environments
	 *        $zonestats is an array with stats for each of the login's zones
	 */
	function TMUDataFetcher($login, $zoneInfo) {

		$this->version = '0.2.1.0';
		$this->zones = $zoneInfo;
		$this->login = strtolower(trim($login));
		$this->getData();
	}  // TMUDataFetcher

	private function getData() {

		$server = new NetRequest('game.trackmaniaunited.com');
		$parser = new ServerResults();

		// all environments (internal & external names)
		$envs = array(
			'Bay' => 'Bay',
			'Coast' => 'Coast',
			'Speed' => 'Desert',
			'Island' => 'Island',
			'Rally' => 'Rally',
			'Alpine' => 'Snow',
			'Stadium' => 'Stadium',
		);

		// bundle player's basic info, world stats & all 7 envs' stats
		// into one request to reduce traffic
		$request = '<?xml version="1.0" encoding="utf-8"?><root><game><name>TmUnited</name><version>' . $this->version . '</version><lang>en</lang></game><author><login></login><password></password><key></key></author><request><name>GetPlayerInfos</name><params><login>' . $this->login . '</login></params></request><request><name>GetLadderStats</name><params><l>' . $this->login . '</l><ld>g</ld><p></p></params></request>';

		foreach ($envs as $envint => $envext) {
			$request .= '<request><name>GetLadderStats</name><params><l>' . $this->login . '</l><ld>' . $envint . '</ld><p></p></params></request>';
		}
		$request .= '</root>';

		if (($result = $server->doPostRequest('/online_game/request.php', $request)) == -1)
			return false;

		$data = $parser->parseXml($result);
		if ($this->login != $data['R']['R'][0]['C'][0]['A'][0])
			return false;

		// store basic info
		$this->nickname = $data['R']['R'][0]['C'][0]['B'][0];
		$this->location = $data['R']['R'][0]['C'][0]['C'][0];

		// store world stats
		$this->wins = $data['R']['R'][1]['C'][0]['J'][0];
		$this->draws = $data['R']['R'][1]['C'][0]['M'][0];
		$this->losses = $data['R']['R'][1]['C'][0]['K'][0];
		$this->lastscore = $data['R']['R'][1]['C'][0]['O'][0];

		$this->totalplayers = $data['R']['R'][1]['C'][0]['P'][0]['A'][0]['C'][0];
		$this->worldrank = $data['R']['R'][1]['C'][0]['P'][0]['A'][0]['D'][0];
		$this->points = $data['R']['R'][1]['C'][0]['P'][0]['A'][0]['E'][0];

		// store all envs' stats
		$i = 2;
		foreach ($envs as $envint => $envext) {
			$this->envstats[$envext] = array(
				'rank' => $data['R']['R'][$i]['C'][0]['P'][0]['A'][0]['D'][0],
				'points' => $data['R']['R'][$i]['C'][0]['P'][0]['A'][0]['E'][0],
				'lastscore' => $data['R']['R'][$i]['C'][0]['O'][0],
				'wins' => $data['R']['R'][$i]['C'][0]['J'][0],
				'draws' => $data['R']['R'][$i]['C'][0]['M'][0],
				'losses' => $data['R']['R'][$i]['C'][0]['K'][0],
			);
			$i++;
		}

		// check zone ladders too?
		if ($this->zones) {
			$size = 40;  // > 48 gives XML errors, so stay on the safe side
			$ppnts = round($this->points);

			// traverse all login's zones except World
			$zones = explode('|', $this->location);
			for ($zindex = 1; $zindex < count($zones); $zindex++) {
				$zone = $zones[0];
				for ($i = 1; $i <= $zindex; $i++) {
					$zone .= '|' . $zones[$i];
				}
				$zrank = $ztotal = $zpnts = '';
				$prank = $ptotal = '';

				// get parent zone's zones
				$parent = substr($zone, 0, strrpos($zone, '|'));
				$done = false;
				$i = 0;
				while (!$done) {

					$request = '<?xml version="1.0" encoding="utf-8"?><root><game><name>TmUnited</name><version>' . $this->version . '</version><lang>en</lang></game><author><login></login><password></password><key></key></author><request><name>GetRankings</name><params><s>' . $i . '</s><l>' . $size . '</l><t>1</t><st>g</st><p>' . $parent . '</p><b>0</b></params></request></root>';
					if (($result = $server->doPostRequest('/online_game/request.php', $request)) == -1)
						return false;

					$data = $parser->parseXml($result);

					foreach ($data['R']['R'][0]['C'][0]['V'] as $zn) {
						if ($zn['N'][0] == $zones[$zindex]) {
							$zpnts = $zn['H'][0];
							$zrank = $zn['G'][0];
							$done = true;
							break;
						}
					}

					$i += $size;
					$ztotal = $data['R']['R'][0]['C'][0]['T'][0];
					if ($i >= $ztotal)
						$done = true;
				}

				// get zone's players
				$done = false;
				$i = 0;
				while (!$done) {

					$request = '<?xml version="1.0" encoding="utf-8"?><root><game><name>TmUnited</name><version>' . $this->version . '</version><lang>en</lang></game><author><login></login><password></password><key></key></author><request><name>GetRankings</name><params><s>' . $i . '</s><l>' . $size . '</l><t>0</t><st>g</st><p>' . $zone . '</p><b>0</b></params></request></root>';
					if (($result = $server->doPostRequest('/online_game/request.php', $request)) == -1)
						return false;

					$data = $parser->parseXml($result);

					foreach ($data['R']['R'][0]['C'][0]['V'] as $pl) {
						if ($pl['N'][0] == $this->nickname && $pl['H'][0] == $ppnts) {
							$prank = $pl['G'][0];
							$done = true;
							break;
						}
					}

					$i += $size;
					$ptotal = $data['R']['R'][0]['C'][0]['T'][0];
					if ($i >= $ptotal)
						$done = true;
					if ($i >= 4000) {  // delimiter to top-4000
						$prank = -1;
						$done = true;
					}
				}

				// store this zone's stats
				$this->zonestats[$zone] = array(
					'zonerank' => $zrank,
					'totalzones' => $ztotal,
					'zonepoints' => $zpnts,
					'playerrank' => $prank,
					'totalplayers' => $ptotal,
					'playerpoints' => $ppnts,
				);
			}
		}
	}  // end getData
}  // end class TMUDataFetcher


// Parser for TMU master server results
// Created by Xymph
class ServerResults {
	var $parser, $struct, $stack;

	// parser functions
	private function openTag($parser, $name, $attribs) {
		//echo 'openTag: ' . $name . "\n";  // print_r($attribs);
		$this->stack[] = $name;
		$this->struct[implode(',',$this->stack)] = '';
	}  // openTag

	private function charData($parser, $data) {
		//echo 'charData: ' . $data . "\n";
		if (trim($data)) {
			$this->struct[implode(',',$this->stack)] .= urldecode($data);
		} elseif ($data == '0') {
			$this->struct[implode(',',$this->stack)] .= '0';
		}
	}  // charData

	private function closeTag($parser, $name) {
		//echo 'closeTag: ' . $name . "\n";
		if (count($this->stack) > 1) {
			$from = implode(',',$this->stack);
			$drop = array_pop($this->stack);  // == $name
			$to = implode(',',$this->stack);
			$this->struct[$to][$name][] = $this->struct[$from];
			unset($this->struct[$from]);
		}
	}  // closeTag

	function parseXml($xml) {

		// clear last results
		$this->struct = array();
		$this->stack = array();

		// create the parser
		$this->parser = xml_parser_create();
		xml_set_object($this->parser, $this);
		xml_set_element_handler($this->parser, 'openTag', 'closeTag');
		xml_set_character_data_handler($this->parser, 'charData');

		// parse xml file
		if (!xml_parse($this->parser, $xml, true)) {
			die(sprintf("XML error: %s at line %d\n",
			            xml_error_string(xml_get_error_code($this->parser)),
			            xml_get_current_line_number($this->parser)));
		}
		xml_parser_free($this->parser);
		return $this->struct;
	}  // parseXml
}  // end class ServerResults

// Simple HTTP Requester
// Created by Jfreu; Updated by Xymph
class NetRequest {

	var $server;

	// Constructor
	function NetRequest($server)
	{
		$this->server = $server;
	}  // NetRequest

	// Simple GET function
	// ok: return string || error: return false || timeout: return -1
	function doRequest($request)
	{
		$fp = @fsockopen($this->server, 80, $errno, $errstr, 4);
		if ($fp) {
			fputs($fp, "GET " . $request .
			           " HTTP/1.0\r\nHost:" . $this->server . "\r\n\r\n");
			stream_set_timeout($fp, 2);
			$res = '';
			$info['timed_out'] = false;
			while (!feof($fp) && !$info['timed_out']) {
				$res .= fread($fp, 512);
				$info = stream_get_meta_data($fp);
			}
			fclose($fp);

			if ($info['timed_out']) {
				return -1;
			} else {
				if (substr($res, 9, 3) != '200')
					return false;
				$page = explode("\r\n\r\n", $res, 2);
				if (!$this->isError($page[1]))
					return trim($page[1]);
			}
		}
		return false;
	}  // doRequest

	// Simple POST function
	// ok: return string || error: return false || timeout: return -1
	function doPostRequest($request, $xml)
	{
		$fp = fsockopen($this->server, 80, $errno, $errstr, 4);
		if ($fp) {
			$size = strlen($xml);
			$request = "POST " . $request . " HTTP/1.1\r\n";
			$request .= "Host: " . $this->server . "\r\n";
			$request .= "Content-Type: application/binary\r\n";
			$request .= "Content-Length: " . $size . "\r\n";
			$request .= "\r\n" . $xml;
			fputs($fp, $request);

			stream_set_timeout($fp, 2);
			$res = '';
			$info['timed_out'] = false;
			while (!feof($fp) && !$info['timed_out']) {
				$res .= fread($fp, 512);
				$info = stream_get_meta_data($fp);
			}
			fclose($fp);
			if ($info['timed_out']) {
				return -1;
			} else {
				if (substr($res, 9, 3) != '200')
					return false;
				$page = explode("\r\n\r\n", $res, 2);
				if (!$this->isError($page[1]))
					return trim($page[1]);
			}
			return true;
		}
		return false;
	}  // doPostRequest

	// Check if the netRequest returns an error
	private function isError($text)
	{
		if (stristr($text, 'ERROR'))
			return true;
		return false;
	}  // isError
}  // end class NetRequest
?>
